@@ -271,6 +271,7 @@ class Worker:
             self.inq, self.outq, self.synq, self.initializer,
             self.initargs, self.maxtasks, self._shutdown, self.on_exit,
             self.sigprotection, self.wrap_exception, self.max_memory_per_child,
+            self.on_ready_counter
         )
 
     def __call__(self):
@@ -1307,7 +1308,7 @@ class Pool:
                 yield worker
 
     def _worker_active(self, worker):
-        for job in values(self._cache):
+        for job in self._cache.values():
             if worker.pid in job.worker_pids():
                 return True
         return False
@@ -1625,7 +1626,7 @@ class Pool:
         debug('pool join complete')
 
     def restart(self):
-        for e in values(self._poolctrl):
+        for e in self._poolctrl.values():
             e.set()
 
     @staticmethod
@@ -1658,7 +1659,12 @@ class Pool:
         debug('helping task handler/workers to finish')
         cls._help_stuff_finish(*help_stuff_finish_args)
 
-        result_handler.terminate()
+        # Send the sentinel to the result handler but don't terminate the
+        # result handler thread. This allows the thread to continue
+        # processing results in ResultHandler.finish_at_shutdown() until
+        # the cache is drained, ensuring that all task results are properly
+        # stored. A call to ResultHandler.terminate() is not necessary here
+        # because the thread will exit naturally when the cache becomes empty.
         cls._set_result_sentinel(outqueue, pool)
 
         if timeout_handler is not None:
@@ -1741,8 +1747,8 @@ class ApplyResult:
         cache[self._job] = self
 
     def __repr__(self):
-        return '<%s: {id} ack:{ack} ready:{ready}>'.format(
-            self.__class__.__name__,
+        return '<{name}: {id} ack:{ack} ready:{ready}>'.format(
+            name=self.__class__.__name__,
             id=self._job, ack=self._accepted, ready=self.ready(),
         )
 
@@ -1850,13 +1856,8 @@ class ApplyResult:
                 except Exception:
                     response = NACK
                     # ignore other errors
-                finally:
-                    if self._send_ack and synqW_fd:
-                        return self._send_ack(
-                            response, pid, self._job, synqW_fd
-                        )
             if self._send_ack and synqW_fd:
-                self._send_ack(response, pid, self._job, synqW_fd)
+                return self._send_ack(response, pid, self._job, synqW_fd)
 
 #
 # Class whose instances are returned by `Pool.map_async()`
